@@ -346,6 +346,10 @@ static PHP_INI_MH(OnSetLogFilter)
 		PG(syslog_filter) = PHP_SYSLOG_FILTER_ASCII;
 		return SUCCESS;
 	}
+	if (!strcmp(filter, "raw")) {
+		PG(syslog_filter) = PHP_SYSLOG_FILTER_RAW;
+		return SUCCESS;
+	}
 
 	return FAILURE;
 }
@@ -1293,7 +1297,7 @@ static ZEND_COLD void php_error_cb(int type, const char *error_filename, const u
 				break;
 			case E_PARSE:
 				error_type_str = "Parse error";
-				syslog_type_int = LOG_EMERG;
+				syslog_type_int = LOG_ERR;
 				break;
 			case E_NOTICE:
 			case E_USER_NOTICE:
@@ -1954,6 +1958,11 @@ void php_request_shutdown(void *dummy)
 		zend_unset_timeout();
 	} zend_end_try();
 
+	/* 17. Deactivate Zend signals */
+#ifdef ZEND_SIGNALS
+	zend_signal_deactivate();
+#endif
+
 #ifdef PHP_WIN32
 	if (PG(com_initialized)) {
 		CoUninitialize();
